<?php

/*********************************************************************

 一行BBSプラグイン (2018/06/17)

 freo.jp
 customize by 32877(http://32877.xii.jp

*********************************************************************/

/* メイン処理 */
function freo_page_linebbs()
{
	global $freo;

	switch ($_REQUEST['freo']['work']) {
		case 'setup':
			freo_page_linebbs_setup();
			break;
		case 'setup_execute':
			freo_page_linebbs_setup_execute();
			break;
		case 'admin':
			freo_page_linebbs_admin();
			break;
		case 'admin_view':
			freo_page_linebbs_admin_view();
			break;
		case 'admin_form':
			freo_page_linebbs_admin_form();
			break;
		case 'admin_post':
			freo_page_linebbs_admin_post();
			break;
		case 'admin_delete':
			freo_page_linebbs_admin_delete();
			break;
		case 'preview':
			freo_page_linebbs_preview();
			break;
		case 'post':
			freo_page_linebbs_post();
			break;
		default:
			freo_page_linebbs_default();
	}

	return;
}

/* セットアップ */
function freo_page_linebbs_setup()
{
	global $freo;

	//ログイン状態確認
	if ($freo->user['authority'] != 'root' and $freo->user['authority'] != 'author') {
		freo_redirect('login', true);
	}

	//リクエストメソッドに応じた処理を実行
	if ($_SERVER['REQUEST_METHOD'] == 'POST') {
		//ワンタイムトークン確認
		if (!freo_token('check')) {
			$freo->smarty->append('errors', '不正なアクセスです。');
		}

		//エラー確認
		if (!$freo->smarty->get_template_vars('errors')) {
			freo_redirect('linebbs/setup_execute?freo%5Btoken%5D=' . freo_token('create'), true);
		}
	}

	//データ割当
	$freo->smarty->assign(array(
		'token'       => freo_token('create'),
		'plugin_id'   => 'linebbs',
		'plugin_name' => FREO_PLUGIN_LINEBBS3287_NAME
	));

	//データ出力
	freo_output('plugins/setup.html');

	return;
}

/* セットアップ | セットアップ実行 */
function freo_page_linebbs_setup_execute()
{
	global $freo;

	//ログイン状態確認
	if ($freo->user['authority'] != 'root' and $freo->user['authority'] != 'author') {
		freo_redirect('login', true);
	}

	//ワンタイムトークン確認
	if (!freo_token('check')) {
		freo_redirect('setup?error=1', true);
	}

	//データベーステーブル存在検証
	if (FREO_DATABASE_TYPE == 'mysql') {
		$query = 'SHOW TABLES';
	} else {
		$query = 'SELECT name FROM sqlite_master WHERE type = \'table\'';
	}
	$stmt = $freo->pdo->query($query);
	if (!$stmt) {
		freo_error($freo->pdo->errorInfo());
	}

	$table = array();
	while ($data = $stmt->fetch(PDO::FETCH_NUM)) {
		$table[$data[0]] = true;
	}

	//データベーステーブル定義
	if (FREO_DATABASE_TYPE == 'mysql') {
		$queries = array(
			'plugin_linebbss' => '(id INT UNSIGNED NOT NULL AUTO_INCREMENT, created DATETIME NOT NULL, modified DATETIME NOT NULL, ip VARCHAR(80) NOT NULL, text TEXT NOT NULL, name TEXT NULL, PRIMARY KEY(id))'
		);
	} else {
		$queries = array(
			'plugin_linebbss' => '(id INTEGER, created DATETIME NOT NULL, modified DATETIME NOT NULL, ip VARCHAR(80) NOT NULL, text TEXT NOT NULL, name TEXT NULL, PRIMARY KEY(id))'
		);
	}

	//データベーステーブル作成
	foreach ($queries as $name => $query) {
		if (empty($table[FREO_DATABASE_PREFIX . $name])) {
			$stmt = $freo->pdo->query('CREATE TABLE ' . FREO_DATABASE_PREFIX . $name . $query);
			if (!$stmt) {
				freo_error($freo->pdo->errorInfo());
			}
		}
	}

	//ログ記録
	freo_log(FREO_PLUGIN_LINEBBS3287_NAME . 'のセットアップを実行しました。');

	//完了画面へ移動
	freo_redirect('linebbs/setup?exec=setup', true);

	return;
}

/* 管理画面 | 一行BBS管理 */
function freo_page_linebbs_admin()
{
	global $freo;

	//ログイン状態確認
	if ($freo->user['authority'] != 'root' and $freo->user['authority'] != 'author') {
		freo_redirect('login', true);
	}

	//パラメータ検証
	if (!isset($_GET['page']) or !preg_match('/^\d+$/', $_GET['page']) or $_GET['page'] < 1) {
		$_GET['page'] = 1;
	}

	//検索条件設定
	$condition = null;
	if (isset($_GET['word'])) {
		$condition .= ' AND text LIKE ' . $freo->pdo->quote('%' . $_GET['word'] . '%');
	}
	if (isset($_GET['name'])) {
		$condition .= ' AND name LIKE ' . $freo->pdo->quote('%' . $_GET['name'] . '%');
	}
	if (isset($_GET['date'])) {
		if (preg_match('/^\d\d\d\d$/', $_GET['date'])) {
			if (FREO_DATABASE_TYPE == 'mysql') {
				$condition .= ' AND DATE_FORMAT(created, \'%Y\') = ' . $freo->pdo->quote($_GET['date']);
			} else {
				$condition .= ' AND STRFTIME(\'%Y\', created) = ' . $freo->pdo->quote($_GET['date']);
			}
		} elseif (preg_match('/^\d\d\d\d\d\d$/', $_GET['date'])) {
			if (FREO_DATABASE_TYPE == 'mysql') {
				$condition .= ' AND DATE_FORMAT(created, \'%Y%m\') = ' . $freo->pdo->quote($_GET['date']);
			} else {
				$condition .= ' AND STRFTIME(\'%Y%m\', created) = ' . $freo->pdo->quote($_GET['date']);
			}
		} elseif (preg_match('/^\d\d\d\d\d\d\d\d$/', $_GET['date'])) {
			if (FREO_DATABASE_TYPE == 'mysql') {
				$condition .= ' AND DATE_FORMAT(created, \'%Y%m%d\') = ' . $freo->pdo->quote($_GET['date']);
			} else {
				$condition .= ' AND STRFTIME(\'%Y%m%d\', created) = ' . $freo->pdo->quote($_GET['date']);
			}
		}
	}
	if ($condition) {
		$condition = ' WHERE id IS NOT NULL ' . $condition;
	}

	//一行BBS取得
	$stmt = $freo->pdo->prepare('SELECT * FROM ' . FREO_DATABASE_PREFIX . 'plugin_linebbss ' . $condition . ' ORDER BY id DESC LIMIT :start, :limit');
	$stmt->bindValue(':start', intval($freo->config['plugin']['linebbs']['admin_limit']) * ($_GET['page'] - 1), PDO::PARAM_INT);
	$stmt->bindValue(':limit', intval($freo->config['plugin']['linebbs']['admin_limit']), PDO::PARAM_INT);
	$flag = $stmt->execute();
	if (!$flag) {
		freo_error($stmt->errorInfo());
	}

	$plugin_linebbss = array();
	while ($data = $stmt->fetch(PDO::FETCH_ASSOC)) {
		$plugin_linebbss[$data['id']] = $data;
	}

	//一行BBS数・ページ数取得
	$stmt = $freo->pdo->query('SELECT COUNT(*) FROM ' . FREO_DATABASE_PREFIX . 'plugin_linebbss ' . $condition);
	if (!$stmt) {
		freo_error($freo->pdo->errorInfo());
	}

	$data                 = $stmt->fetch(PDO::FETCH_NUM);
	$plugin_linebbs_count = $data[0];
	$plugin_linebbs_page  = ceil($plugin_linebbs_count / $freo->config['plugin']['linebbs']['admin_limit']);

	//データ割当
	$freo->smarty->assign(array(
		'token'                => freo_token('create'),
		'plugin_linebbss'      => $plugin_linebbss,
		'plugin_linebbs_count' => $plugin_linebbs_count,
		'plugin_linebbs_page'  => $plugin_linebbs_page
	));

	return;
}

/* 管理画面 | 一行BBS表示 */
function freo_page_linebbs_admin_view()
{
	global $freo;

	//ログイン状態確認
	if ($freo->user['authority'] != 'root' and $freo->user['authority'] != 'author') {
		freo_redirect('login', true);
	}

	//パラメータ検証
	if (!isset($_GET['id']) or !preg_match('/^\d+$/', $_GET['id']) or $_GET['id'] < 1) {
		freo_redirect('linebbs/admin?error=1');
	}

	//一行BBS取得
	$stmt = $freo->pdo->prepare('SELECT * FROM ' . FREO_DATABASE_PREFIX . 'plugin_linebbss WHERE id = :id');
	$stmt->bindValue(':id', $_GET['id'], PDO::PARAM_INT);
	$flag = $stmt->execute();
	if (!$flag) {
		freo_error($stmt->errorInfo());
	}

	if ($data = $stmt->fetch(PDO::FETCH_ASSOC)) {
		$plugin_linebbs = $data;
	} else {
		freo_error('指定された投稿が見つかりません。');
	}

	//データ割当
	$freo->smarty->assign(array(
		'token'          => freo_token('create'),
		'plugin_linebbs' => $plugin_linebbs
	));

	return;
}

/* 管理画面 | 一行BBS入力 */
function freo_page_linebbs_admin_form()
{
	global $freo;

	//ログイン状態確認
	if ($freo->user['authority'] != 'root' and $freo->user['authority'] != 'author') {
		freo_redirect('login', true);
	}

	//パラメータ検証
	if (!isset($_GET['id']) or !preg_match('/^\d+$/', $_GET['id']) or $_GET['id'] < 1) {
		freo_redirect('linebbs/admin?error=1');
	}

	//リクエストメソッドに応じた処理を実行
	if ($_SERVER['REQUEST_METHOD'] == 'POST') {
		//ワンタイムトークン確認
		if (!freo_token('check')) {
			$freo->smarty->append('errors', '不正なアクセスです。');
		}

		//入力データ検証
		if (!$freo->smarty->get_template_vars('errors')) {
			//本文
			if ($_POST['plugin_linebbs']['text'] == '') {
				$freo->smarty->append('errors', '本文が入力されていません。');
			} elseif (mb_strlen($_POST['plugin_linebbs']['text'], 'UTF-8') > 5000) {
				$freo->smarty->append('errors', '本文は5000文字以内で入力してください。');
			}
		}

		//エラー確認
		if ($freo->smarty->get_template_vars('errors')) {
			//エラー表示
			$plugin_linebbs = $_POST['plugin_linebbs'];
		} else {
			$_SESSION['input'] = $_POST;

			//登録処理へ移動
			freo_redirect('linebbs/admin_post?freo%5Btoken%5D=' . freo_token('create') . '&id=' . $_GET['id']);
		}
	} else {
		//編集データ取得
		$stmt = $freo->pdo->prepare('SELECT * FROM ' . FREO_DATABASE_PREFIX . 'plugin_linebbss WHERE id = :id');
		$stmt->bindValue(':id', $_GET['id'], PDO::PARAM_INT);
		$flag = $stmt->execute();
		if (!$flag) {
			freo_error($stmt->errorInfo());
		}

		if ($data = $stmt->fetch(PDO::FETCH_ASSOC)) {
			$plugin_linebbs = $data;
		} else {
			freo_error('指定された投稿が見つかりません。');
		}
	}

	//データ割当
	$freo->smarty->assign(array(
		'token' => freo_token('create'),
		'input' => array(
			'plugin_linebbs' => $plugin_linebbs,
		)
	));

	return;
}

/* 管理画面 | 一行BBS登録 */
function freo_page_linebbs_admin_post()
{
	global $freo;

	//ログイン状態確認
	if ($freo->user['authority'] != 'root' and $freo->user['authority'] != 'author') {
		freo_redirect('login', true);
	}

	//入力データ確認
	if (empty($_SESSION['input'])) {
		freo_redirect('linebbs/admin?error=1');
	}

	//ワンタイムトークン確認
	if (!freo_token('check')) {
		freo_redirect('linebbs/admin?error=1');
	}

	//入力データ取得
	$linebbs = $_SESSION['input']['plugin_linebbs'];

	//データ登録
	$stmt = $freo->pdo->prepare('UPDATE ' . FREO_DATABASE_PREFIX . 'plugin_linebbss SET modified = :now, text = :text, name = :name WHERE id = :id');
	$stmt->bindValue(':now',  date('Y-m-d H:i:s'));
	$stmt->bindValue(':text', $linebbs['text']);
	$stmt->bindValue(':name', $linebbs['name']);
	$stmt->bindValue(':id',   $linebbs['id'], PDO::PARAM_INT);
	$flag = $stmt->execute();
	if (!$flag) {
		freo_error($stmt->errorInfo());
	}

	//入力データ破棄
	$_SESSION['input'] = array();

	//ログ記録
	freo_log('一行BBSの投稿を編集しました。');

	//一行BBS管理へ移動
	freo_redirect('linebbs/admin?exec=update&id=' . $linebbs['id']);

	return;
}

/* 管理画面 | 一行BBS投稿削除 */
function freo_page_linebbs_admin_delete()
{
	global $freo;

	//ログイン状態確認
	if ($freo->user['authority'] != 'root' and $freo->user['authority'] != 'author') {
		freo_redirect('login', true);
	}

	//パラメータ検証
	if (!isset($_GET['id']) or !preg_match('/^\d+$/', $_GET['id']) or $_GET['id'] < 1) {
		freo_redirect('linebbs/admin?error=1');
	}

	//ワンタイムトークン確認
	if (!freo_token('check')) {
		freo_redirect('linebbs/admin?error=1');
	}

	//一行BBS削除
	$stmt = $freo->pdo->prepare('DELETE FROM ' . FREO_DATABASE_PREFIX . 'plugin_linebbss WHERE id = :id');
	$stmt->bindValue(':id', $_GET['id'], PDO::PARAM_INT);
	$flag = $stmt->execute();
	if (!$flag) {
		freo_error($stmt->errorInfo());
	}

	//連番リセット
	if (FREO_DATABASE_TYPE == 'mysql') {
		$stmt = $freo->pdo->query('ALTER TABLE ' . FREO_DATABASE_PREFIX . 'plugin_linebbss AUTO_INCREMENT = 0');
		if (!$stmt) {
			freo_error($freo->pdo->errorInfo());
		}
	}

	//ログ記録
	freo_log('一行BBSを削除しました。');

	//一行BBS管理へ移動
	freo_redirect('linebbs/admin?exec=delete&id=' . $_GET['id']);

	return;
}

/* 一行BBS確認 */
function freo_page_linebbs_preview()
{
	global $freo;

	//入力データ確認
	if (empty($_SESSION['input'])) {
		freo_redirect('linebbs');
	}

	//リクエストメソッドに応じた処理を実行
	if ($_SERVER['REQUEST_METHOD'] == 'POST') {
		//ワンタイムトークン確認
		if (!freo_token('check')) {
			freo_redirect('linebbs');
		}

		//登録処理へ移動
		freo_redirect('linebbs/post?freo%5Btoken%5D=' . freo_token('create'));
	}

	//データ割当
	$freo->smarty->assign(array(
		'token'          => freo_token('create'),
		'plugin_linebbs' => $_SESSION['input']['plugin_linebbs']
	));

	return;
}

/* 一行BBS登録 */
function freo_page_linebbs_post()
{
	global $freo;

	//入力データ確認
	if (empty($_SESSION['input'])) {
		freo_redirect('linebbs');
	}

	//ワンタイムトークン確認
	if (!freo_token('check')) {
		freo_redirect('linebbs');
	}

	//入力データ取得
	$linebbs = $_SESSION['input']['plugin_linebbs'];

	//データ登録
	$stmt = $freo->pdo->prepare('INSERT INTO ' . FREO_DATABASE_PREFIX . 'plugin_linebbss VALUES(NULL, :now1, :now2, :ip, :text, :name)');
	$stmt->bindValue(':now1', date('Y-m-d H:i:s'));
	$stmt->bindValue(':now2', date('Y-m-d H:i:s'));
	$stmt->bindValue(':ip',   $_SERVER['REMOTE_ADDR']);
	$stmt->bindValue(':text', $linebbs['text']);
	$stmt->bindValue(':name', $linebbs['name']);
	$flag = $stmt->execute();
	if (!$flag) {
		freo_error($stmt->errorInfo());
	}

	//入力データ破棄
	$_SESSION['input'] = array();

	//ログ記録
	freo_log('一行BBSを新規に登録しました。');

	//登録完了画面へ移動
	freo_redirect('linebbs/default');

	return;
}

/* 一行BBS入力 */
function freo_page_linebbs_default()
{
	global $freo;

	//パラメータ検証
	if (!isset($_GET['page']) or !preg_match('/^\d+$/', $_GET['page']) or $_GET['page'] < 1) {
		$_GET['page'] = 1;
	}

	//検索条件設定
	$condition = null;
	if (isset($_GET['word'])) {
		$condition .= ' AND text LIKE ' . $freo->pdo->quote('%' . $_GET['word'] . '%');
	}
	if (isset($_GET['name'])) {
		$condition .= ' AND name LIKE ' . $freo->pdo->quote('%' . $_GET['name'] . '%');
	}
	if (isset($_GET['date'])) {
		if (preg_match('/^\d\d\d\d$/', $_GET['date'])) {
			if (FREO_DATABASE_TYPE == 'mysql') {
				$condition .= ' AND DATE_FORMAT(created, \'%Y\') = ' . $freo->pdo->quote($_GET['date']);
			} else {
				$condition .= ' AND STRFTIME(\'%Y\', created) = ' . $freo->pdo->quote($_GET['date']);
			}
		} elseif (preg_match('/^\d\d\d\d\d\d$/', $_GET['date'])) {
			if (FREO_DATABASE_TYPE == 'mysql') {
				$condition .= ' AND DATE_FORMAT(created, \'%Y%m\') = ' . $freo->pdo->quote($_GET['date']);
			} else {
				$condition .= ' AND STRFTIME(\'%Y%m\', created) = ' . $freo->pdo->quote($_GET['date']);
			}
		} elseif (preg_match('/^\d\d\d\d\d\d\d\d$/', $_GET['date'])) {
			if (FREO_DATABASE_TYPE == 'mysql') {
				$condition .= ' AND DATE_FORMAT(created, \'%Y%m%d\') = ' . $freo->pdo->quote($_GET['date']);
			} else {
				$condition .= ' AND STRFTIME(\'%Y%m%d\', created) = ' . $freo->pdo->quote($_GET['date']);
			}
		}
	}
	if ($condition) {
		$condition = ' WHERE id IS NOT NULL ' . $condition;
	}

	//一行BBS取得
	$stmt = $freo->pdo->prepare('SELECT * FROM ' . FREO_DATABASE_PREFIX . 'plugin_linebbss ' . $condition . ' ORDER BY id DESC LIMIT :start, :limit');
	$stmt->bindValue(':start', intval($freo->config['plugin']['linebbs']['admin_limit']) * ($_GET['page'] - 1), PDO::PARAM_INT);
	$stmt->bindValue(':limit', intval($freo->config['plugin']['linebbs']['admin_limit']), PDO::PARAM_INT);
	$flag = $stmt->execute();
	if (!$flag) {
		freo_error($stmt->errorInfo());
	}

	$plugin_linebbss = array();
	while ($data = $stmt->fetch(PDO::FETCH_ASSOC)) {
		$plugin_linebbss[$data['id']] = $data;
	}

	//一行BBS数・ページ数取得
	$stmt = $freo->pdo->query('SELECT COUNT(*) FROM ' . FREO_DATABASE_PREFIX . 'plugin_linebbss ' . $condition);
	if (!$stmt) {
		freo_error($freo->pdo->errorInfo());
	}

	$data                 = $stmt->fetch(PDO::FETCH_NUM);
	$plugin_linebbs_count = $data[0];
	$plugin_linebbs_page  = ceil($plugin_linebbs_count / $freo->config['plugin']['linebbs']['admin_limit']);

	//データ割当
	$freo->smarty->assign(array(
		'token'                => freo_token('create'),
		'plugin_linebbss'      => $plugin_linebbss,
		'plugin_linebbs_count' => $plugin_linebbs_count,
		'plugin_linebbs_page'  => $plugin_linebbs_page
	));

	//リクエストメソッドに応じた処理を実行
	if ($_SERVER['REQUEST_METHOD'] == 'POST') {
		//ワンタイムトークン確認
		if (!freo_token('check')) {
			$freo->smarty->append('errors', '不正なアクセスです。');
		}

		//入力データ検証
		if (!$freo->smarty->get_template_vars('errors')) {
			//本文
			if ($_POST['plugin_linebbs']['text'] == '') {
				$freo->smarty->append('errors', '本文が入力されていません。');
			} elseif (mb_strlen($_POST['plugin_linebbs']['text'], 'UTF-8') > 5000) {
				$freo->smarty->append('errors', '本文は5000文字以内で入力してください。');
			}
		}

		//エラー確認
		if ($freo->smarty->get_template_vars('errors')) {
			//エラー表示
			$plugin_linebbs = $_POST['plugin_linebbs'];
		} else {
			$_SESSION['input'] = $_POST;

			if (isset($_POST['preview'])) {
				//プレビューへ移動
				freo_redirect('linebbs/preview');
			} else {
				//登録処理へ移動
				freo_redirect('linebbs/post?freo%5Btoken%5D=' . freo_token('create'));
			}
		}
	} else {
		if (!empty($_GET['session']) and !empty($_SESSION['input'])) {
			//入力データ復元
			$plugin_linebbs = $_SESSION['input']['plugin_linebbs'];
		} else {
			//新規データ設定
			$plugin_linebbs = array();
		}
	}

	//データ割当
	$freo->smarty->assign(array(
		'token' => freo_token('create'),
		'input' => array(
			'plugin_linebbs' => $plugin_linebbs
		)
	));


	return;
}

?>
